module Instruction where

data Instruction = 
    Operand0 String |
    OperandInt1 String Int |
    OperandInt2 String Int Int |
    OperandString1 String String |
    FlagResetDontCare |
    FlagResetRegistersSafe |
    Label String deriving Eq

instance Show Instruction where
     show (Operand0 ins) = ins
     show (OperandInt1 ins op1) = ins ++ " " ++ hexChar op1
     show (OperandString1 ins op1) = ins ++ " " ++ op1
     show (Label l) = l ++ ":"
     show FlagResetDontCare = "[FlagResetDontCare]"
     show FlagResetRegistersSafe = "[FlagResetRegistersSafe]"

hexChar i | i < 16 = ["0123456789ABCDEF" !! i]
hexChar i = error ("invalid integer " ++ show i)

hexString 0 0 = ""
hexString n 0 = "0" ++ hexString (n-1) 0
hexString n i = hexString (n - 1) (div i 16) ++ hexChar (mod i 16)


isInstruction (Label _) = False
isInstruction _ = True

isLabel (Label _) = True
isLabel _ = False

isBreakRegisterA (Operand0 "KA") = True
isBreakRegisterA (Operand0 "CH") = True
isBreakRegisterA (Operand0 "CY") = True
isBreakRegisterA (Operand0 "MA") = True
isBreakRegisterA (Operand0 "M+") = True
isBreakRegisterA (Operand0 "M-") = True
isBreakRegisterA (OperandInt1 "TIA" _) = True
isBreakRegisterA (OperandInt1 "AIA" _) = True
isBreakRegisterA (OperandString1 "CAL" "INPUT") = True
isBreakRegisterA (OperandString1 "CAL" "CMPL") = True
isBreakRegisterA (OperandString1 "CAL" "CHNG") = True
isBreakRegisterA (OperandString1 "CAL" "SIFT") = True
isBreakRegisterA _ = False

isBreakRegisterY (Operand0 "CH") = True
isBreakRegisterY (Operand0 "CY") = True
isBreakRegisterY (OperandInt1 "TIY" _) = True
isBreakRegisterY (OperandInt1 "AIY" _) = True
isBreakRegisterY (OperandString1 "CAL" "CHNG") = True
isBreakRegisterY (OperandString1 "CAL" "DEM-") = True
isBreakRegisterY (OperandString1 "CAL" "DEM+") = True
isBreakRegisterY _ = False

isResetFlag (Operand0 "KA") = False
isResetFlag (Operand0 "M+") = False
isResetFlag (Operand0 "M-") = False
isResetFlag (OperandInt1 "AIA" _) = False
isResetFlag (OperandInt1 "AIY" _) = False
isResetFlag (OperandInt1 "CIA" _) = False
isResetFlag (OperandInt1 "CIY" _) = False
isResetFlag (OperandString1 "CAL" "SIFT") = False
isResetFlag (Label _) = False
isResetFlag _ = True

isBreakMemory (Operand0 "AM") = True
isBreakMemory (OperandString1 "CAL" "DEM-") = True
isBreakMemory (OperandString1 "CAL" "DEM+") = True
isBreakMemory _ = False

instructionSize (Operand0 _) = 1
instructionSize (OperandInt1 _ _) = 2
instructionSize (OperandString1 "JUMP" _) = 3
instructionSize (OperandString1 "CAL" _) = 2
instructionSize (Label _) = 0
instructionSize (FlagResetDontCare) = 0
instructionSize (FlagResetRegistersSafe) = 1

{-
   Ar Yr
E0 NC NC
E1 NC NC
E2 NC NC
E3 C  NC
E4 C  NC
E5 C  C
E6 C  NC
E7 NC NC
E8 NC NC
E9 NC NC
EA NC NC
EB NC NC
EC NC NC
ED NC NC
EE NC C
EF NC C
-}
